'use strict';

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

var _jsdom = require('jsdom');

var _jsdom2 = _interopRequireDefault(_jsdom);

var _xmlserializer = require('xmlserializer');

var _xmlserializer2 = _interopRequireDefault(_xmlserializer);

var _d3SankeyDiagram = require('d3-sankey-diagram');

var _d3SankeyDiagram2 = _interopRequireDefault(_d3SankeyDiagram);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// diagram

_commander2.default.arguments('<file>').option('-s, --size <w>,<h>', 'width and height', parseSize).option('-m, --margins <n>[,...]', '1, 2 or 4 margin values', parseMargins).action(function (filename) {
  _fs2.default.readFile(filename, 'utf8', function (err, data) {
    if (err) throw err; // we'll not consider error handling for now
    var parsedData = JSON.parse(data);
    var svg = drawDiagram(parsedData);
    process.stdout.write(svg);
  });
}).parse(process.argv);

function parseMargins(val) {
  val = val.split(',').map(function (x) {
    return x.trim();
  }).map(Number);
  if (val.length === 1) {
    return { top: val[0], right: val[0], bottom: val[0], left: val[0] };
  } else if (val.length === 2) {
    return { top: val[0], right: val[1], bottom: val[0], left: val[1] };
  } else if (val.length === 4) {
    return { top: val[0], right: val[1], bottom: val[2], left: val[3] };
  } else {
    throw new Error('Expected 1, 2 or 4 numbers');
  }
}

function parseSize(val) {
  val = val.split(',').map(function (x) {
    return x.trim();
  }).map(Number);
  if (val.length === 1) {
    return [val[0], val[0]];
  } else if (val.length === 2) {
    return val;
  } else {
    throw new Error('Expected 1 or 2');
  }
}

function drawDiagram(data) {
  var width = _commander2.default.size ? _commander2.default.size[0] : 800,
      height = _commander2.default.size ? _commander2.default.size[1] : 600;

  var color = _d2.default.scale.category20();

  var diagram = (0, _d3SankeyDiagram2.default)().duration(null).width(width).height(height).nodeTitle(function (d) {
    return d.data.title !== undefined ? d.data.title : d.id;
  }).linkTypeTitle(function (d) {
    return d.data.title;
  }).linkColor(function (d) {
    return d.data.color !== undefined ? d.data.color : color(d.data.type);
  });

  if (_commander2.default.margins) {
    diagram.margins(_commander2.default.margins);
  }

  var document = _jsdom2.default.jsdom();
  var el = _d2.default.select(document).select('body').append('div');

  el.datum(data).call(diagram);

  // put default styles inline
  el.select('svg').attr('viewBox', '0 0 ' + width + ' ' + height).style('font-family', '"Helvetica Neue", Helvetica, Arial, sans-serif');

  el.selectAll('.link').style('opacity', 0.8);

  el.selectAll('line').style('stroke', function (d) {
    return style(d) === 'process' ? '#888' : '#000';
  }).style('stroke-width', function (d) {
    return style(d) === 'process' ? '4px' : '1px';
  });

  el.selectAll('rect').style('fill', 'none');

  el.selectAll('.group').select('rect').style('fill', '#eee').style('stroke', '#bbb').style('stroke-width', '0.5');

  el.selectAll('.group').select('text').style('fill', '#999');

  // add background
  el.select('svg').insert('rect', ':first-child').attr('width', width).attr('height', height).style('fill', 'white');

  // create a file blob of our SVG.
  var svg = serialize(el.select('svg').node());
  return svg;
}

function style(d) {
  return (d.data || {}).style;
}

var serialize = function serialize(node) {
  return '<?xml version="1.0" standalone="no"?>' + '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">' + _xmlserializer2.default.serializeToString(node);
};
